/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package device;

import java.util.HashMap;
import interfaces.DeviceDefinition;
import app.DeviceInfoWindow;
import app.SESimGUI;

/**
 * Device
 * 
 * Abstract class for defined devices
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public abstract class Device implements DeviceDefinition {	
	private String name_ = "";
	private HashMap<String, String> bitmap_ = null;
	private DeviceInfoWindow diw_ = null;
	
	private boolean activate_ = false;
	private float duration_ = -1.0f;
	private int priority_ = -1;
	private String location_ = null;
	private String alt_ = null;
	private float fade_ = -1.0f;
	private String autoExtraction_ = null;
	
	/**
	 * Constructor
	 */
	public Device() {
		bitmap_ = new HashMap<String, String>();
	}
	
	/**
	 * Constructor
	 * 
	 * @param diw DeviceInfoWindow to use
	 */
	public Device(DeviceInfoWindow diw) {
		bitmap_ = new HashMap<String, String>();
		diw_ = diw;
	}
	
	/**
	 * Method for retrieving the DeviceInfoWindow
	 * @return DeviceInfoWindow
	 */
	public DeviceInfoWindow getDeviceInfoWindow() {
		return diw_;
	}
	
	/**
	 * Method for retrieving the name of the device
	 * @return String with device
	 */
	public String getName() {
		return name_;
	}
	
	/**
	 * Method for setting the name
	 * @param name String with the name
	 */
	public void setName(String name) {
		name_ = name;
	}
	
	/**
	 * Method for adding a bitmap to the device
	 * @param field GUI-String to be mapped to
	 * @param bitmap String with path and name to the bitmap
	 */
	public void addBitmap(String field, String bitmap) {
		bitmap_.put(field, bitmap);
	}
	
	/**
	 * Method for retrieving the assigned bitmaps
	 * @return HashMap<String, String> with the bitmaps
	 */
	public HashMap<String, String> getBitmaps() {
		return bitmap_;
	}

	/**
	 * Method for playing the device
	 * @param settings String[] with settings
	 * @param gui SESimGUI for output
	 * @return true if no error occured else false
	 */
	public abstract boolean playDevice(String[] settings, SESimGUI gui);
	
	/**
	 * Method for resetting the device settings
	 * @param gui SESimGUI for output
	 */
	public abstract void resetSettings(SESimGUI gui);
	
	/**
	 * Method for pausing the device
	 */
	public abstract void pause();
	
	/**
	 * Method for resuming the device
	 */
	public abstract void resume();
	
	/**
	 * Method for retrieving the activation status
	 * @return true if activated else false
	 */
	public boolean isActivate() {
		return activate_;
	}

	/**
	 * Sets the active mode for the device
	 * @param activate true if activated else false
	 */
	public void setActivate(boolean activate) {
		this.activate_ = activate;
	}
	
	public String getAlt() {
		return alt_;
	}

	public String getAutoExtraction() {
		return autoExtraction_;
	}

	public float getDuration() {
		return duration_;
	}

	public float getFade() {
		return fade_;
	}

	public String getLocation() {
		return location_;
	}

	public int getPriority() {
		return priority_;
	}

	public void setAlt(String alt) {
		alt_ = alt;
	}

	public void setAutoExtraction(String autoExtraction) {
		autoExtraction_ = autoExtraction;
	}

	public void setDuration(float duration) {
		duration_ =  duration;
	}

	public void setFade(float fade) {
		fade_ = fade;
	}

	public void setLocation(String location) {
		location_ = location;
	}

	public void setPriority(int priority) {
		priority_ = priority;
	}
	
	public void resetDeviceValues() {
		activate_ = false;
		duration_ = -1.0f;
		priority_ = -1;
		location_ = "";
		alt_ = "";
		fade_ = -1.0f;
		autoExtraction_ = "";
	}
}
