/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package device;

import interfaces.DeviceDefinition;

import java.awt.Color;
import java.util.Timer;

import javax.swing.border.EtchedBorder;

import lib.ClassificationScheme;
import lib.HelperFunctions;
import lib.SEMDefinitions;

import app.DeviceInfoWindow;
import app.SESimGUI;
import timer.FanTask;


/**
 * FanDevice
 * 
 * Class which represents the fan device
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class FanDevice extends Device {
	private final static String INTENSITYVALUE = "intensity-value";
	private final static String INTENSITYRANGE = "intensity-range";
	private float intensityValue_ = 0.0f;
	private String intensityRange_ = "";
	private float oldIntensityValueLeft_ = -1.0f;
	private float oldIntensityValueRight_ = -1.0f;
	
	private FanTask taskLeft_ = null;
	private FanTask taskRight_ = null;
	
	/**
	 * Constructor
	 * @param diw DeviceInfoWindow
	 */
	public FanDevice(DeviceInfoWindow diw) {
		super(diw);
		setName("FanDevice");
		
		addBitmap("panFanLeft", "images/fanleft.gif");
		addBitmap("panFanRight", "images/fanright.gif");
	}

	/**
	 * Method for retrieving a string representation
	 * @return String with representation
	 */
	public String toString() {
		return "FanDevice";
	}

	/**
	 * Method for reseting device settings
	 * @param gui SESimGUI
	 */
	public void resetSettings(SESimGUI gui) {
		if (getLocation() != null &&
				ClassificationScheme.containsLocation(getLocation())) {
			// NOTE: only X values are relevant and there also only left, right, and everywhere
			if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
				setOldIntensityValueLeft(intensityValue_);
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				if (gui != null) {
					gui.panFanLeft.setBorder(new EtchedBorder());
					gui.panFanLeft.drawText("");
					getDeviceInfoWindow().removeValue("FanDevice Left");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
				setOldIntensityValueRight(intensityValue_);
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}
				
				if (gui != null) {
					gui.panFanRight.setBorder(new EtchedBorder());
					gui.panFanRight.drawText("");
					getDeviceInfoWindow().removeValue("FanDevice Right");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
				setOldIntensityValueLeft(intensityValue_);
				setOldIntensityValueRight(intensityValue_);
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}
				
				if (gui != null) {
					gui.panFanLeft.setBorder(new EtchedBorder());
					gui.panFanRight.setBorder(new EtchedBorder());
					gui.panFanLeft.drawText("");
					gui.panFanRight.drawText("");
					getDeviceInfoWindow().removeValue("FanDevice Left");
					getDeviceInfoWindow().removeValue("FanDevice Right");
				}
			}
		}
		else {
			setOldIntensityValueLeft(intensityValue_);	
			setOldIntensityValueRight(intensityValue_);
			
			if (taskLeft_ != null) {
				taskLeft_.cancel();
				taskLeft_ = null;
			}
			
			if (taskRight_ != null) {
				taskRight_.cancel();
				taskRight_ = null;
			}
			
			if (gui != null) {
				gui.panFanLeft.setBorder(new EtchedBorder());
				gui.panFanRight.setBorder(new EtchedBorder());
				gui.panFanLeft.drawText("");
				gui.panFanRight.drawText("");
				getDeviceInfoWindow().removeValue("FanDevice Left");
				getDeviceInfoWindow().removeValue("FanDevice Right");
			}
		}
		
		intensityValue_ = 0.0f;
		intensityRange_ = "";
		
		resetDeviceValues();
	}
	
	/**
	 * Method for playing the device
	 * @param settings String[] with device settings
	 * @param gui SESimGUI
	 * @return true if everything is ok else false
	 */
	public boolean playDevice(String[] settings, SESimGUI gui) {	
		resetSettings(null);
		
		for (int i = 0; i < settings.length; i++) {
			String setting = settings[i].trim();
			String[] parts = setting.split("=");
			if (parts[0].equals(INTENSITYVALUE))
				setIntensityValue(Float.valueOf(parts[1]));
			else if (parts[0].equals(INTENSITYRANGE))
				setIntensityRange(parts[1]);
			else if (parts[0].equals(SEMDefinitions.ACTIVATE)) {
				if (parts[1].equals(SEMDefinitions.FALSE))
					setActivate(false);
				else
					setActivate(true);
			}
			else if (parts[0].equals(SEMDefinitions.ALT))
				setAlt(parts[1]);
			else if (parts[0].equals(SEMDefinitions.AUTOEXTRACTION))
				setAutoExtraction(parts[1]);
			else if (parts[0].equals(SEMDefinitions.DURATION))
				setDuration(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.FADE))
				setFade(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.LOCATION))
				setLocation(parts[1]);
			else if (parts[0].equals(SEMDefinitions.PRIORITY))
				setPriority(Integer.valueOf(parts[1]));
			else {
				System.out.println("[" + getName() + "] Unknown attribute: " + parts[0] + "! Aborting!");
				return false;
			}
		}
		
		if (getIntensityRange() != null && !getIntensityRange().equals("")) {
			if (!HelperFunctions.isInIntensityRange(getIntensityValue(), getIntensityRange(), getName()))
				return false;
		}
		
		if (getFade() > -1 && getDuration() > -1) {
			if (getFade() > getDuration()) {
				System.out.println("[" + getName() + "] Fade cannot be bigger than the duration of effect! Aborting!");
				return false;
			}
		}
		
		if (getDeviceInfoWindow() != null) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				// NOTE: only X values are relevant and there also only left, right, and everywhere
				if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
					getDeviceInfoWindow().changeValue("FanDevice Left", "intensity-value: " + getIntensityValue() + 
							", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
							", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
							", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					getDeviceInfoWindow().changeValue("FanDevice Right", "intensity-value: " + getIntensityValue() + 
							", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
							", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
							", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
					getDeviceInfoWindow().changeValue("FanDevice Left", "intensity-value: " + getIntensityValue() + 
							", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
							", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
							", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					getDeviceInfoWindow().changeValue("FanDevice Right", "intensity-value: " + getIntensityValue() + 
							", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
							", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
							", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				}
			}
			else { // automatically everywhere is taken if no location is given
				getDeviceInfoWindow().changeValue("FanDevice Left", "intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				getDeviceInfoWindow().changeValue("FanDevice Right", "intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
			}
		}
		
		if (isActivate() || getDuration() > 0 || getFade() > 0) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				// NOTE: only X values are relevant and there also only left, right, and everywhere
				if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
					gui.panFanLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmLeft = new Timer();
					taskLeft_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueLeft(), gui, true, this);
					tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					gui.panFanRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmRight = new Timer();
					taskRight_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueRight(), gui, false, this);
					tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
					gui.panFanLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					gui.panFanRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmLeft = new Timer();
					taskLeft_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueLeft(), gui, true, this);
					tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
					
					Timer tmRight = new Timer();
					taskRight_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueRight(), gui, false, this);
					tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
				}
			}
			else {
				gui.panFanLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui.panFanRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
				Timer tmLeft = new Timer();
				taskLeft_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueLeft(), gui, true, this);
				tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
				
				Timer tmRight = new Timer();
				taskRight_ = new FanTask(getDeviceInfoWindow(), getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValueRight(), gui, false, this);
				tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
			}
		}
		else if (!isActivate()) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				// NOTE: only X values are relevant and there also only left, right, and everywhere
				if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
					gui.panFanLeft.setBorder(new EtchedBorder());
					gui.panFanLeft.drawText("");
					oldIntensityValueLeft_ = -1.0f;
					if (taskLeft_ != null) {
						taskLeft_.cancel();
						taskLeft_ = null;
					}
					
					getDeviceInfoWindow().changeValue("FanDevice Left", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					gui.panFanRight.setBorder(new EtchedBorder());
					gui.panFanRight.drawText("");
					oldIntensityValueRight_ = -1.0f;
					
					if (taskRight_ != null) {
						taskRight_.cancel();
						taskRight_ = null;
					}
					getDeviceInfoWindow().changeValue("FanDevice Right", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
					gui.panFanLeft.setBorder(new EtchedBorder());
					gui.panFanRight.setBorder(new EtchedBorder());
					gui.panFanLeft.drawText("");
					gui.panFanRight.drawText("");
					oldIntensityValueLeft_ = -1.0f;
					oldIntensityValueRight_ = -1.0f;
					
					if (taskRight_ != null) {
						taskRight_.cancel();
						taskRight_ = null;
					}
					
					if (taskLeft_ != null) {
						taskLeft_.cancel();
						taskLeft_ = null;
					}
					
					getDeviceInfoWindow().changeValue("FanDevice Left", "");
					getDeviceInfoWindow().changeValue("FanDevice Right", "");
				}
			}
			else {
				gui.panFanLeft.setBorder(new EtchedBorder());
				gui.panFanRight.setBorder(new EtchedBorder());
				gui.panFanLeft.drawText("");
				gui.panFanRight.drawText("");
				oldIntensityValueLeft_ = -1.0f;
				oldIntensityValueRight_ = -1.0f;
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				getDeviceInfoWindow().changeValue("FanDevice Left", "");
				getDeviceInfoWindow().changeValue("FanDevice Right", "");
			}
		}
		
		return true;
	}		

	/**
	 * Method for pausing the device
	 */
	public void pause() {
		if (taskLeft_ != null)
			taskLeft_.pause();
		
		if (taskRight_ != null)
			taskRight_.pause();
	}

	/**
	 * Method for resuming the device
	 */
	public void resume() {
		if (taskLeft_ != null)
			taskLeft_.resume();
		
		if (taskRight_ != null)
			taskRight_.resume();
	}
	
	/**
	 * Method for getting the intensity value in percent
	 * @return float with intensity value
	 */
	public float getIntensityValue() {
		return intensityValue_;
	}
	
	/**
	 * Method for setting the intensity value in percent
	 * @param intensityValue float with the intensity value
	 */
	public void setIntensityValue(float intensityValue) {
		intensityValue_ = intensityValue;
	}
	
	/**
	 * Method for getting the intensity range
	 * @return String with intensity range
	 */
	public String getIntensityRange() {
		return intensityRange_;
	}
	
	/**
	 * Method for setting the intensity range
	 * @param intensityRange String with the intensity range
	 */
	public void setIntensityRange(String intensityRange) {
		intensityRange_ = intensityRange;
	}
	
	public void setOldIntensityValueLeft(float oldIntensityValue) {
		this.oldIntensityValueLeft_ = oldIntensityValue;
	}

	public float getOldIntensityValueLeft() {
		return oldIntensityValueLeft_;
	}
	
	public void setOldIntensityValueRight(float oldIntensityValue) {
		this.oldIntensityValueRight_ = oldIntensityValue;
	}

	public float getOldIntensityValueRight() {
		return oldIntensityValueRight_;
	}
}
