/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package device;

import interfaces.DeviceDefinition;

import java.awt.Color;
import java.util.Timer;

import javax.swing.border.EtchedBorder;

import org.iso.mpeg.mpegv.sedl.AutoExtractionType;

import lib.ClassificationScheme;
import lib.HelperFunctions;
import lib.SEMDefinitions;

import timer.LightTask;

import app.SESimGUI;
import app.DeviceInfoWindow;

/**
 * LightDevice
 * 
 * Class which represents the light device
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class LightDevice extends Device {	
	private final static String INTENSITYVALUE = "intensity-value";
	private final static String INTENSITYRANGE = "intensity-range";
	private final static String COLOR = "color";
	
	private Color color_ = Color.WHITE;
	private float intensityValue_ = -1.0f;
	private String intensityRange_ = "";
	private float oldIntensityValueLeft_ = -1.0f;
	private float oldIntensityValueCenterLeft_ = -1.0f;
	private float oldIntensityValueCenter_ = -1.0f;
	private float oldIntensityValueCenterRight_ = -1.0f;
	private float oldIntensityValueRight_ = -1.0f;
	
	private LightTask taskLeft_ = null;
	private LightTask taskCenterLeft_ = null;
	private LightTask taskCenter_ = null;
	private LightTask taskCenterRight_ = null;
	private LightTask taskRight_ = null;

	/**
	 * Constructor
	 * @param diw DeviceInfoWindow
	 */
	public LightDevice(DeviceInfoWindow diw) {
		super(diw);
		setName("LightDevice");
		
		addBitmap("panLightLeft", "images/lightleft.gif");
		addBitmap("panLightBackLeft", "images/lightbackleft.gif");
		addBitmap("panLightBackMiddle", "images/lightbackmiddle.gif");
		addBitmap("panLightBackRight", "images/lightbackright.gif");
		addBitmap("panLightRight", "images/lightright.gif");
	}
	
	/**
	 * Method for retrieving a string representation
	 * @return String with representation
	 */
	public String toString() {
		return "LightDevice";
	}
	
	/**
	 * Method for reseting device settings
	 * @param gui SESimGUI
	 */
	public void resetSettings(SESimGUI gui) {
		if (getLocation() != null &&
				ClassificationScheme.containsLocation(getLocation())) {
			// NOTE: only X values are relevant and there also only left, centerleft, center, centerright, right, and everywhere
			if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
				setOldIntensityValueLeft(intensityValue_);
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				if (gui != null) {
					gui.panLightLeft.setBorder(new EtchedBorder());
					gui.panLightLeft.drawText("", "", "");
					getDeviceInfoWindow().removeValue("LightDevice Left");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
				setOldIntensityValueCenterLeft(intensityValue_);
				
				if (taskCenterLeft_ != null) {
					taskCenterLeft_.cancel();
					taskCenterLeft_ = null;
				}
				
				if (gui != null) {
					gui.panLightBackLeft.setBorder(new EtchedBorder());
					gui.panLightBackLeft.drawText("");
					getDeviceInfoWindow().removeValue("LightDevice LeftBack");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
				setOldIntensityValueCenter(intensityValue_);
				
				if (taskCenter_ != null) {
					taskCenter_.cancel();
					taskCenter_ = null;
				}				
				
				if (gui != null) {
					gui.panLightBackMiddle.setBorder(new EtchedBorder());
					gui.panLightBackMiddle.drawText("");
					getDeviceInfoWindow().removeValue("LightDevice MiddleBack");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
				setOldIntensityValueCenterRight(intensityValue_);
				
				if (taskCenterRight_ != null) {
					taskCenterRight_.cancel();
					taskCenterRight_ = null;
				}
				
				if (gui != null) {
					gui.panLightBackRight.setBorder(new EtchedBorder());
					gui.panLightBackRight.drawText("");
					getDeviceInfoWindow().removeValue("LightDevice RightBack");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
				setOldIntensityValueRight(intensityValue_);
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}
				
				if (gui != null) {
					gui.panLightRight.setBorder(new EtchedBorder());
					gui.panLightRight.drawText("", "", "");
					getDeviceInfoWindow().removeValue("LightDevice Right");
				}
			}
			else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
				setOldIntensityValueLeft(intensityValue_);
				setOldIntensityValueCenterLeft(intensityValue_);
				setOldIntensityValueCenter(intensityValue_);
				setOldIntensityValueCenterRight(intensityValue_);
				setOldIntensityValueRight(intensityValue_);
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				if (taskCenterLeft_ != null) {
					taskCenterLeft_.cancel();
					taskCenterLeft_ = null;
				}
				
				if (taskCenter_ != null) {
					taskCenter_.cancel();
					taskCenter_ = null;
				}
				
				if (taskCenterRight_ != null) {
					taskCenterRight_.cancel();
					taskCenterRight_ = null;
				}
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}
				
				
				if (gui != null) {
					gui.panLightLeft.setBorder(new EtchedBorder());
					gui.panLightBackLeft.setBorder(new EtchedBorder());
					gui.panLightBackMiddle.setBorder(new EtchedBorder());
					gui.panLightBackRight.setBorder(new EtchedBorder());
					gui.panLightRight.setBorder(new EtchedBorder());
					gui.panLightLeft.drawText("", "", "");
					gui.panLightBackLeft.drawText("");
					gui.panLightBackMiddle.drawText("");
					gui.panLightBackRight.drawText("");
					gui.panLightRight.drawText("", "", "");
					getDeviceInfoWindow().removeValue("LightDevice Left");
					getDeviceInfoWindow().removeValue("LightDevice Right");
					getDeviceInfoWindow().removeValue("LightDevice LeftBack");
					getDeviceInfoWindow().removeValue("LightDevice RightBack");
					getDeviceInfoWindow().removeValue("LightDevice MiddleBack");
				}
			}
		}
		else {
			setOldIntensityValueLeft(intensityValue_);
			setOldIntensityValueCenterLeft(intensityValue_);
			setOldIntensityValueCenter(intensityValue_);
			setOldIntensityValueCenterRight(intensityValue_);
			setOldIntensityValueRight(intensityValue_);
			
			if (taskLeft_ != null) {
				taskLeft_.cancel();
				taskLeft_ = null;
			}
			
			if (taskCenterLeft_ != null) {
				taskCenterLeft_.cancel();
				taskCenterLeft_ = null;
			}
			
			if (taskCenter_ != null) {
				taskCenter_.cancel();
				taskCenter_ = null;
			}
			
			if (taskCenterRight_ != null) {
				taskCenterRight_.cancel();
				taskCenterRight_ = null;
			}
			
			if (taskRight_ != null) {
				taskRight_.cancel();
				taskRight_ = null;
			}
			
			
			if (gui != null) {
				gui.panLightLeft.setBorder(new EtchedBorder());
				gui.panLightBackLeft.setBorder(new EtchedBorder());
				gui.panLightBackMiddle.setBorder(new EtchedBorder());
				gui.panLightBackRight.setBorder(new EtchedBorder());
				gui.panLightRight.setBorder(new EtchedBorder());
				gui.panLightLeft.drawText("", "", "");
				gui.panLightBackLeft.drawText("");
				gui.panLightBackMiddle.drawText("");
				gui.panLightBackRight.drawText("");
				gui.panLightRight.drawText("", "", "");
				getDeviceInfoWindow().removeValue("LightDevice Left");
				getDeviceInfoWindow().removeValue("LightDevice Right");
				getDeviceInfoWindow().removeValue("LightDevice LeftBack");
				getDeviceInfoWindow().removeValue("LightDevice RightBack");
				getDeviceInfoWindow().removeValue("LightDevice MiddleBack");
			}
		}
				
		color_ = Color.WHITE;
		intensityValue_ = -1.0f;
		intensityRange_ = "";
		
		resetDeviceValues();
	}
	
	/**
	 * Method for playing the device
	 * @param settings String[] with device settings
	 * @param gui SESimGUI
	 * @return true if everything ok else false
	 */
	public boolean playDevice(String[] settings, SESimGUI gui) {
		resetSettings(null);
				
		for (int i = 0; i < settings.length; i++) {
			String setting = settings[i].trim();
			String[] parts = setting.split("=");
			if (parts[0].equals(INTENSITYVALUE))
				setIntensityValue(Float.valueOf(parts[1]));
			else if (parts[0].equals(INTENSITYRANGE))
				setIntensityRange(parts[1]);
			else if (parts[0].equals(COLOR)) {
				Color c = ClassificationScheme.getRGBforColor(parts[1]);
				if (c == null) // then we should have a hexadecimal value
					c = HelperFunctions.hexToRGB(parts[1]);
					
				setColor(c);
			}
			else if (parts[0].equals(SEMDefinitions.ACTIVATE)) {
				if (parts[1].equals(SEMDefinitions.FALSE))
					setActivate(false);
				else
					setActivate(true);
			}
			else if (parts[0].equals(SEMDefinitions.ALT))
				setAlt(parts[1]);
			else if (parts[0].equals(SEMDefinitions.AUTOEXTRACTION))
				setAutoExtraction(parts[1]);
			else if (parts[0].equals(SEMDefinitions.DURATION))
				setDuration(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.FADE))
				setFade(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.LOCATION))
				setLocation(parts[1]);
			else if (parts[0].equals(SEMDefinitions.PRIORITY))
				setPriority(Integer.valueOf(parts[1]));
			else {
				System.out.println("[" + getName() + "] Unknown attribute: " + parts + "! Aborting!");
				return false;
			}
		}
		
		if (getIntensityRange() != null && !getIntensityRange().equals("")) {
			if (!HelperFunctions.isInIntensityRange(getIntensityValue(), getIntensityRange(), getName()))
				return false;
		}
		
		if (getFade() > -1 && getDuration() > -1) {
			if (getFade() > getDuration()) {
				System.out.println("[" + getName() + "] Fade cannot be bigger than the duration of effect! Aborting!");
				return false;
			}
		}
		
		if (getDeviceInfoWindow() != null) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				if (getLocation() != null &&
						ClassificationScheme.containsLocation(getLocation())) {
					// NOTE: only X values are relevant and there also only left, centerleft, center, centerright, right, and everywhere
					if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
						getDeviceInfoWindow().changeValue("LightDevice Left", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					}
					else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
						getDeviceInfoWindow().changeValue("LightDevice LeftBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					}
					else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
						getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());			
					}
					else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
						getDeviceInfoWindow().changeValue("LightDevice RightBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					}
					else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
						getDeviceInfoWindow().changeValue("LightDevice Right", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					}
					else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
						getDeviceInfoWindow().changeValue("LightDevice Left", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
						getDeviceInfoWindow().changeValue("LightDevice Right", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
						getDeviceInfoWindow().changeValue("LightDevice LeftBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
						getDeviceInfoWindow().changeValue("LightDevice RightBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
						getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "color: R" + getColor().getRed() +
								"G" + getColor().getGreen() + "B" + getColor().getBlue() +
								", intensity-value: " + getIntensityValue() + 
								", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
								", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
								", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
					}
				}		
			}
			else {
				getDeviceInfoWindow().changeValue("LightDevice Left", "color: R" + getColor().getRed() +
						"G" + getColor().getGreen() + "B" + getColor().getBlue() +
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				getDeviceInfoWindow().changeValue("LightDevice Right", "color: R" + getColor().getRed() +
						"G" + getColor().getGreen() + "B" + getColor().getBlue() +
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				getDeviceInfoWindow().changeValue("LightDevice LeftBack", "color: R" + getColor().getRed() +
						"G" + getColor().getGreen() + "B" + getColor().getBlue() +
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				getDeviceInfoWindow().changeValue("LightDevice RightBack", "color: R" + getColor().getRed() +
						"G" + getColor().getGreen() + "B" + getColor().getBlue() +
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
				getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "color: R" + getColor().getRed() +
						"G" + getColor().getGreen() + "B" + getColor().getBlue() +
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());		
			}
		}
		
		boolean autoColor = false;
		if (getAutoExtraction() != null && (getAutoExtraction().equalsIgnoreCase(AutoExtractionType.VISUAL.toString())
				|| getAutoExtraction().equalsIgnoreCase(AutoExtractionType.BOTH.toString()))) {
			autoColor = true;
		}
		
		float[] range = null;
		if (getIntensityRange() != null) {
			String[] splitRange = getIntensityRange().split(" ");
			if (splitRange.length == 2) {
				range = new float[2];
				range[0] = Float.valueOf(splitRange[0]);
				range[1] = Float.valueOf(splitRange[1]);
			}
		}
		
		if (isActivate() || getDuration() > 0 || getFade() > 0) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				// NOTE: only X values are relevant and there also only left, centerleft, center, centerright, right, and everywhere
				if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
					gui.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmLeft = new Timer();
					taskLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueLeft(), gui, this);
					
					// for color calculation every half second is enough
					if (autoColor)
						tmLeft.scheduleAtFixedRate(taskLeft_, 0, 500);
					else
						tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
					gui.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmCenterLeft = new Timer();
					taskCenterLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterLeft(), gui, this);

					// for color calculation every half second is enough
					if (autoColor)
						tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, 500);
					else
						tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
					gui.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmCenter = new Timer();
					taskCenter_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenter(), gui, this);

					// for color calculation every half second is enough
					if (autoColor)
						tmCenter.scheduleAtFixedRate(taskCenter_, 0, 500);
					else
						tmCenter.scheduleAtFixedRate(taskCenter_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
					gui.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmCenterRight = new Timer();
					taskCenterRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterRight(), gui, this);

					// for color calculation every half second is enough
					if (autoColor)
						tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, 500);
					else
						tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					gui.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					Timer tmRight = new Timer();
					taskRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueRight(), gui, this);

					// for color calculation every half second is enough
					if (autoColor)
						tmRight.scheduleAtFixedRate(taskRight_, 0, 500);
					else
						tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
					gui.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					gui.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					gui.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
					gui.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					gui.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					
					Timer tmLeft = new Timer();
					Timer tmCenterLeft = new Timer();
					Timer tmCenter = new Timer();
					Timer tmCenterRight = new Timer();
					Timer tmRight = new Timer();
					
					taskLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueLeft(), gui, this);
					taskCenterLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterLeft(), gui, this);
					taskCenter_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenter(), gui, this);
					taskCenterRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterRight(), gui, this);
					taskRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueRight(), gui, this);

					// for color calculation every half second is enough
					if (autoColor) {
						tmLeft.scheduleAtFixedRate(taskLeft_, 0, 500);
						tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, 500);
						tmCenter.scheduleAtFixedRate(taskCenter_, 0, 500);
						tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, 500);
						tmRight.scheduleAtFixedRate(taskRight_, 0, 500);
					}
					else {
						tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
						tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, DeviceDefinition.TIMERINTERVAL);
						tmCenter.scheduleAtFixedRate(taskCenter_, 0, DeviceDefinition.TIMERINTERVAL);
						tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, DeviceDefinition.TIMERINTERVAL);
						tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
					}
				}
			}
			else {
				gui.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
				
				Timer tmLeft = new Timer();
				Timer tmCenterLeft = new Timer();
				Timer tmCenter = new Timer();
				Timer tmCenterRight = new Timer();
				Timer tmRight = new Timer();
				
				taskLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueLeft(), gui, this);
				taskCenterLeft_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterLeft(), gui, this);
				taskCenter_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenter(), gui, this);
				taskCenterRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueCenterRight(), gui, this);
				taskRight_ = new LightTask(getDeviceInfoWindow(), getColor(), autoColor, getIntensityValue(), getDuration(), (long)(getFade()*10), range, getLocation(), getOldIntensityValueRight(), gui, this);

				// for color calculation every half second is enough
				if (autoColor) {
					tmLeft.scheduleAtFixedRate(taskLeft_, 0, 500);
					tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, 500);
					tmCenter.scheduleAtFixedRate(taskCenter_, 0, 500);
					tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, 500);
					tmRight.scheduleAtFixedRate(taskRight_, 0, 500);
				}
				else {
					tmLeft.scheduleAtFixedRate(taskLeft_, 0, DeviceDefinition.TIMERINTERVAL);
					tmCenterLeft.scheduleAtFixedRate(taskCenterLeft_, 0, DeviceDefinition.TIMERINTERVAL);
					tmCenter.scheduleAtFixedRate(taskCenter_, 0, DeviceDefinition.TIMERINTERVAL);
					tmCenterRight.scheduleAtFixedRate(taskCenterRight_, 0, DeviceDefinition.TIMERINTERVAL);
					tmRight.scheduleAtFixedRate(taskRight_, 0, DeviceDefinition.TIMERINTERVAL);
				}
			}				
		}
		else if (!isActivate()) {
			if (getLocation() != null &&
					ClassificationScheme.containsLocation(getLocation())) {
				// NOTE: only X values are relevant and there also only left, centerleft, center, centerright, right, and everywhere
				if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
					gui.panLightLeft.setBorder(new EtchedBorder());
					gui.panLightLeft.drawText("");
					oldIntensityValueLeft_ = -1.0f;
					if (taskLeft_ != null) {
						taskLeft_.cancel();
						taskLeft_ = null;
					}
					
					getDeviceInfoWindow().changeValue("LightDevice Left", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
					gui.panLightBackLeft.setBorder(new EtchedBorder());
					gui.panLightBackLeft.drawText("");
					oldIntensityValueCenterLeft_ = -1.0f;
					if (taskCenterLeft_ != null) {
						taskCenterLeft_.cancel();
						taskCenterLeft_ = null;
					}
					
					getDeviceInfoWindow().changeValue("LightDevice LeftBack", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
					gui.panLightBackMiddle.setBorder(new EtchedBorder());
					gui.panLightBackMiddle.drawText("");
					oldIntensityValueCenter_ = -1.0f;
					if (taskCenter_ != null) {
						taskCenter_.cancel();
						taskCenter_ = null;
					}
					
					getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "");				
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
					gui.panLightBackRight.setBorder(new EtchedBorder());
					gui.panLightBackRight.drawText("");
					oldIntensityValueCenterRight_ = -1.0f;
					if (taskCenterRight_ != null) {
						taskCenterRight_.cancel();
						taskCenterRight_ = null;
					}
					
					getDeviceInfoWindow().changeValue("LightDevice RightBack", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					gui.panLightRight.setBorder(new EtchedBorder());
					gui.panLightRight.drawText("");
					oldIntensityValueRight_ = -1.0f;
					if (taskRight_ != null) {
						taskRight_.cancel();
						taskRight_ = null;
					}
					
					getDeviceInfoWindow().changeValue("LightDevice Right", "");
				}
				else if (getLocation().startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
					gui.panLightLeft.setBorder(new EtchedBorder());
					gui.panLightLeft.drawText("");
					gui.panLightBackLeft.setBorder(new EtchedBorder());
					gui.panLightBackLeft.drawText("");
					gui.panLightBackMiddle.setBorder(new EtchedBorder());
					gui.panLightBackMiddle.drawText("");
					gui.panLightBackRight.setBorder(new EtchedBorder());
					gui.panLightBackRight.drawText("");
					gui.panLightRight.setBorder(new EtchedBorder());
					gui.panLightRight.drawText("");
					
					oldIntensityValueLeft_ = -1.0f;
					oldIntensityValueCenterLeft_ = -1.0f;
					oldIntensityValueCenter_ = -1.0f;
					oldIntensityValueCenterRight_ = -1.0f;
					oldIntensityValueRight_ = -1.0f;					
					
					if (taskLeft_ != null) {
						taskLeft_.cancel();
						taskLeft_ = null;
					}
					
					if (taskCenterLeft_ != null) {
						taskCenterLeft_.cancel();
						taskCenterLeft_ = null;
					}
					
					if (taskCenter_ != null) {
						taskCenter_.cancel();
						taskCenter_ = null;
					}
					
					if (taskCenterRight_ != null) {
						taskCenterRight_.cancel();
						taskCenterRight_ = null;
					}					
					
					if (taskRight_ != null) {
						taskRight_.cancel();
						taskRight_ = null;
					}					
					
					getDeviceInfoWindow().changeValue("LightDevice Left", "");
					getDeviceInfoWindow().changeValue("LightDevice LeftBack", "");
					getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "");
					getDeviceInfoWindow().changeValue("LightDevice RightBack", "");					
					getDeviceInfoWindow().changeValue("LightDevice Right", "");
				}
			}
			else {
				gui.panLightLeft.setBorder(new EtchedBorder());
				gui.panLightLeft.drawText("");
				gui.panLightBackLeft.setBorder(new EtchedBorder());
				gui.panLightBackLeft.drawText("");
				gui.panLightBackMiddle.setBorder(new EtchedBorder());
				gui.panLightBackMiddle.drawText("");
				gui.panLightBackRight.setBorder(new EtchedBorder());
				gui.panLightBackRight.drawText("");
				gui.panLightRight.setBorder(new EtchedBorder());
				gui.panLightRight.drawText("");
				
				oldIntensityValueLeft_ = -1.0f;
				oldIntensityValueCenterLeft_ = -1.0f;
				oldIntensityValueCenter_ = -1.0f;
				oldIntensityValueCenterRight_ = -1.0f;
				oldIntensityValueRight_ = -1.0f;					
				
				if (taskLeft_ != null) {
					taskLeft_.cancel();
					taskLeft_ = null;
				}
				
				if (taskCenterLeft_ != null) {
					taskCenterLeft_.cancel();
					taskCenterLeft_ = null;
				}
				
				if (taskCenter_ != null) {
					taskCenter_.cancel();
					taskCenter_ = null;
				}
				
				if (taskCenterRight_ != null) {
					taskCenterRight_.cancel();
					taskCenterRight_ = null;
				}					
				
				if (taskRight_ != null) {
					taskRight_.cancel();
					taskRight_ = null;
				}					
				
				getDeviceInfoWindow().changeValue("LightDevice Left", "");
				getDeviceInfoWindow().changeValue("LightDevice LeftBack", "");
				getDeviceInfoWindow().changeValue("LightDevice MiddleBack", "");
				getDeviceInfoWindow().changeValue("LightDevice RightBack", "");					
				getDeviceInfoWindow().changeValue("LightDevice Right", "");
			}
		}
		
		return true;
	}

	/**
	 * Retrieves the color (if set)
	 * @return Color with the color
	 */
	public Color getColor() {
		return color_;
	}

	/**
	 * Sets the color
	 * @param color Color with the color
	 */
	public void setColor(Color color) {
		this.color_ = color;
	}

	/**
	 * Method for pausing the device
	 */
	public void pause() {
		if (taskLeft_ != null)
			taskLeft_.pause();
		
		if (taskCenterLeft_ != null)
			taskCenterLeft_.pause();
		
		if (taskCenter_ != null)
			taskCenter_.pause();
		
		if (taskCenterRight_ != null)
			taskCenterRight_.pause();
		
		if (taskRight_ != null)
			taskRight_.pause();
	}

	/**
	 * Method for resuming the device
	 */
	public void resume() {		
		if (taskLeft_ != null)
			taskLeft_.resume();
		
		if (taskCenterLeft_ != null)
			taskCenterLeft_.resume();
		
		if (taskCenter_ != null)
			taskCenter_.resume();
		
		if (taskCenterRight_ != null)
			taskCenterRight_.resume();
		
		if (taskRight_ != null)
			taskRight_.resume();
	}
	
	/**
	 * Method for getting the intensity value in percent
	 * @return float with intensity value
	 */
	public float getIntensityValue() {
		return intensityValue_;
	}
	
	/**
	 * Method for setting the intensity value in percent
	 * @param intensityValue float with the intensity value
	 */
	public void setIntensityValue(float intensityValue) {
		intensityValue_ = intensityValue;
	}
	
	/**
	 * Method for getting the intensity range
	 * @return String with intensity range
	 */
	public String getIntensityRange() {
		return intensityRange_;
	}
	
	/**
	 * Method for setting the intensity range
	 * @param intensityRange String with the intensity range
	 */
	public void setIntensityRange(String intensityRange) {
		intensityRange_ = intensityRange;
	}
	
	public void setOldIntensityValueLeft(float oldIntensityValue) {
		this.oldIntensityValueLeft_ = oldIntensityValue;
	}

	public float getOldIntensityValueLeft() {
		return oldIntensityValueLeft_;
	}
	
	public void setOldIntensityValueCenterLeft(float oldIntensityValue) {
		this.oldIntensityValueCenterLeft_ = oldIntensityValue;
	}

	public float getOldIntensityValueCenterLeft() {
		return oldIntensityValueCenterLeft_;
	}
	
	public void setOldIntensityValueCenter(float oldIntensityValue) {
		this.oldIntensityValueCenter_ = oldIntensityValue;
	}

	public float getOldIntensityValueCenter() {
		return oldIntensityValueCenter_;
	}
	
	public void setOldIntensityValueCenterRight(float oldIntensityValue) {
		this.oldIntensityValueCenterRight_ = oldIntensityValue;
	}

	public float getOldIntensityValueCenterRight() {
		return oldIntensityValueCenterRight_;
	}
	
	public void setOldIntensityValueRight(float oldIntensityValue) {
		this.oldIntensityValueRight_ = oldIntensityValue;
	}

	public float getOldIntensityValueRight() {
		return oldIntensityValueRight_;
	}
}
