/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package device;

import interfaces.DeviceDefinition;

import java.awt.Color;
import java.util.Timer;

import javax.swing.border.EtchedBorder;

import lib.HelperFunctions;
import lib.SEMDefinitions;

import timer.WaterSprayerTask;
import app.SESimGUI;
import app.DeviceInfoWindow;

/**
 * WaterSprayerDevice
 * 
 * Class which represents the water sprayer device
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class WaterSprayerDevice extends Device {
	private final static String INTENSITYVALUE = "intensity-value";
	private final static String INTENSITYRANGE = "intensity-range";
	private final static String SPRAYINGTYPE = "sprayingType";
	private String sprayingType_ = "";
	private String intensityRange_ = "";
	private float intensityValue_ = 0.0f;
	private float oldIntensityValue_ = -1.0f;
	
	private WaterSprayerTask task_ = null;
	
	/**
	 * Constructor
	 * @param diw DeviceInfoWindow
	 */
	public WaterSprayerDevice(DeviceInfoWindow diw) {
		super(diw);
		setName("WaterSprayerDevice");
		addBitmap("panWaterSprayer", "images/watersprayer.gif");
	}
	
	/**
	 * Method for retrieving a string representation
	 * @return String with representation
	 */
	public String toString() {
		return "WaterSprayerDevice";
	}
	
	/**
	 * Method for reseting device settings
	 * @param gui SESimGUI
	 */
	public void resetSettings(SESimGUI gui) {
		resetDeviceValues();
		setOldIntensityValue(intensityValue_);
		intensityValue_ = 0.0f;
		intensityRange_ = "";
		
		if (task_ != null) {
			task_.cancel();
			task_ = null;
		}
		
		if (gui != null) {
			gui.panWaterSprayer.setBorder(new EtchedBorder());
			gui.panWaterSprayer.drawText("");
			getDeviceInfoWindow().removeValue("WaterSprayerDevice");
		}
	}

	/**
	 * Method for playing the device
	 * @param settings String[] with device settings
	 * @param gui SESimGUI
	 * @return true if everything ok else false
	 */
	public boolean playDevice(String[] settings, SESimGUI gui) {
		resetSettings(null);
		
		for (int i = 0; i < settings.length; i++) {
			String setting = settings[i].trim();
			String[] parts = setting.split("=");
			if (parts[0].equals(SPRAYINGTYPE))
				setSprayingType(parts[1]);
			if (parts[0].equals(INTENSITYVALUE))
				setIntensityValue(Float.valueOf(parts[1]));
			else if (parts[0].equals(INTENSITYRANGE))
				setIntensityRange(parts[1]);
			else if (parts[0].equals(SEMDefinitions.ACTIVATE)) {
				if (parts[1].equals(SEMDefinitions.FALSE)) {
					gui.panWaterSprayer.setBorder(new EtchedBorder());
					setActivate(false);
				}
				else {
					gui.panWaterSprayer.setBorder(new EtchedBorder(Color.RED, Color.RED));
					setActivate(true);
				}
			}
			else if (parts[0].equals(SEMDefinitions.ALT))
				setAlt(parts[1]);
			else if (parts[0].equals(SEMDefinitions.AUTOEXTRACTION))
				setAutoExtraction(parts[1]);
			else if (parts[0].equals(SEMDefinitions.DURATION))
				setDuration(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.FADE))
				setFade(Float.valueOf(parts[1]));
			else if (parts[0].equals(SEMDefinitions.LOCATION))
				setLocation(parts[1]);
			else if (parts[0].equals(SEMDefinitions.PRIORITY))
				setPriority(Integer.valueOf(parts[1]));
			else {
				System.out.println("[" + getName() + "] Unknown attribute: " + parts[0] + "! Aborting!");
				return false;
			}
		}
		
		if (getIntensityRange() != null && !getIntensityRange().equals("")) {
			if (!HelperFunctions.isInIntensityRange(getIntensityValue(), getIntensityRange(), getName()))
				return false;
		}
		
		if (getDeviceInfoWindow() != null) {
			if (isActivate())
				getDeviceInfoWindow().changeValue("WaterSprayerDevice", "sprayingType: " + getSprayingType() + 
						", intensity-value: " + getIntensityValue() + 
						", intensity-range: " + getIntensityRange() + ", duration: " + getDuration() + 
						", fade: " + getFade() + ", priority: " + getPriority() + ", location: " + getLocation() + 
						", alt: " + getAlt() + ", autoExtraction: " + getAutoExtraction());
			else
				getDeviceInfoWindow().changeValue("WaterSprayerDevice", "");
		}

		if (isActivate() || getDuration() > 0 || getFade() > 0) {
			Timer tm = new Timer();
			task_ = new WaterSprayerTask(getIntensityValue(), getDuration(), (long)(getFade()*10), getOldIntensityValue(), gui, this);
			tm.scheduleAtFixedRate(task_, 0, DeviceDefinition.TIMERINTERVAL);
		}
		else if (!isActivate()) {
			if (task_ != null) {
				task_.cancel();
				task_ = null;
			}
			
			oldIntensityValue_ = -1.0f;
			
			gui.panWaterSprayer.drawText("");
		}
		
		return true;
	}
	
	/**
	 * Method for pausing the device
	 */
	public void pause() {
		if (task_ != null)
			task_.pause();
	}

	/**
	 * Method for resuming the device
	 */
	public void resume() {
		if (task_ != null)
			task_.resume();
	}
	
	/**
	 * Method for getting the intensity value in percent
	 * @return float with intensity value
	 */
	public float getIntensityValue() {
		return intensityValue_;
	}
	
	/**
	 * Method for setting the intensity value in percent
	 * @param intensityValue float with the intensity value
	 */
	public void setIntensityValue(float intensityValue) {
		intensityValue_ = intensityValue;
	}
	
	/**
	 * Method for getting the intensity range
	 * @return String with intensity range
	 */
	public String getIntensityRange() {
		return intensityRange_;
	}
	
	/**
	 * Method for setting the intensity range
	 * @param intensityRange String with the intensity range
	 */
	public void setIntensityRange(String intensityRange) {
		intensityRange_ = intensityRange;
	}

	public void setSprayingType(String sprayingType_) {
		this.sprayingType_ = sprayingType_;
	}

	public String getSprayingType() {
		return sprayingType_;
	}

	public void setOldIntensityValue(float oldIntensityValue) {
		this.oldIntensityValue_ = oldIntensityValue;
	}

	public float getOldIntensityValue() {
		return oldIntensityValue_;
	}
}
