/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.awt.Color;

/**
 * Functions
 * 
 * Class which represents a collection of various functions
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class HelperFunctions {
	/**
	 * Method for checking if the intensity value is within the intensity range
	 * @param value intensity value
	 * @param range intensity range
	 * @param deviceName name of the device
	 * @return true if it is in range else false
	 */
	public static boolean isInIntensityRange(float value, String range, String deviceName) {
		if (range != null) {
			String[] intensityRange = range.split(" ");
			if (intensityRange.length == 2) {
				if (Float.parseFloat(intensityRange[0]) > value ||
						Float.parseFloat(intensityRange[1]) < value) {
					System.out.println("[" + deviceName + "] Intensity value is outside of the intensity range! Aborting!");
					return false;
				}
			}
			else
				System.out.println("[" + deviceName + "] Intensity range is not given correctly!");
		}
		
		return true;
	}
	
	/**
	 * Method for rounding to a given precision
	 * @param number to round
	 * @param precision count of digits after dot
	 * @return rounded number
	 */
	public static double round(double number, int precision)
	{
		number *= Math.pow(10.0, precision);
		if (number >= 0)
			number = Math.floor(number + 0.5);
		else
			number = Math.ceil(number - 0.5);

		number /= Math.pow(10.0, precision);

		return number;
	}
	
	public static Color hexToRGB(String hex) {
		// input: #ffffff
		if (hex == null || hex.length() != 7)
			return Color.black;
		
		hex = hex.substring(1); // removing #
		
		int r = Integer.parseInt(hex.substring(0, 2), 16);
		int g = Integer.parseInt(hex.substring(2, 4), 16);
		int b = Integer.parseInt(hex.substring(4, 6), 16);
		
		return new Color(r, g, b);
	}
	
	/**
	 *  Create a RGB Color object based on this HSL color space with a different
	 *  Luminance value. The percent specified is an absolute value.
	 *
	 *	@param hsl the HSL color object
	 *  @paran percent the Luminance value between 0 - 100
	 *  @return the RGB Color object
	 */
	public static Color adjustLuminance(float[] hsl, float percent)
	{
		if (hsl[1] <0.0f || hsl[1] > 100.0f)
		{
			System.out.println("Saturation outside of expected range");
			return null;
		}

		if (percent <0.0f || percent > 100.0f)
		{
			System.out.println("Luminance outside of expected range");
			return null;
		}

		hsl[0] = hsl[0] % 360.0f;
		hsl[0] /= 360f;
		hsl[1] /= 100f;
		percent /= 100f;

		float q = 0;

		if (percent < 0.5)
			q = percent * (1 + hsl[1]);
		else
			q = (percent + hsl[1]) - (hsl[1] * percent);

		float p = 2 * percent - q;

		float r = Math.max(0, HueToRGB(p, q, hsl[0] + (1.0f / 3.0f)));
		float g = Math.max(0, HueToRGB(p, q, hsl[0]));
		float b = Math.max(0, HueToRGB(p, q, hsl[0] - (1.0f / 3.0f)));

		return new Color(r, g, b);
	}
	
	private static float HueToRGB(float p, float q, float h)
	{
		if (h < 0) h += 1;

		if (h > 1 ) h -= 1;

		if (6 * h < 1)
		{
			return p + ((q - p) * 6 * h);
		}

		if (2 * h < 1 )
		{
			return  q;
		}

		if (3 * h < 2)
		{
			return p + ( (q - p) * 6 * ((2.0f / 3.0f) - h) );
		}

   		return p;
	}

	/**
	 *  Convert a RGB Color to it corresponding HSL values.
	 *
	 *  @return float[] with HSL values.
	 */
	public static float[] fromRGB(Color color)
	{
		float[] rgb = color.getRGBColorComponents( null );
		float r = rgb[0];
		float g = rgb[1];
		float b = rgb[2];

		float min = Math.min(r, Math.min(g, b));
		float max = Math.max(r, Math.max(g, b));

		float h = 0;
		if (max == min)
			h = 0;
		else if (max == r)
			h = ((60 * (g - b) / (max - min)) + 360) % 360;
		else if (max == g)
			h = (60 * (b - r) / (max - min)) + 120;
		else if (max == b)
			h = (60 * (r - g) / (max - min)) + 240;

		float l = (max + min) / 2;

		float s = 0;
		if (max == min)
			s = 0;
		else if (l <= .5f)
			s = (max - min) / (max + min);
		else
			s = (max - min) / (2 - max - min);

		return new float[] {h, s * 100, l * 100};
	}
}
