/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;
import org.cybergarage.upnp.Action;
import org.cybergarage.upnp.Argument;
import org.cybergarage.upnp.ArgumentList;
import org.cybergarage.upnp.ControlPoint;
import org.cybergarage.upnp.Device;
import org.cybergarage.upnp.Service;
import org.cybergarage.upnp.ServiceList;
import org.cybergarage.upnp.UPnP;
import org.cybergarage.upnp.device.DeviceChangeListener;
import org.upnp.ClassType;
import org.upnp.ContainerType;
import org.upnp.DIDLLite;
import org.upnp.DescType;
import org.upnp.GenreType;
import org.upnp.ItemType;
import org.upnp.PersonWithRoleType;
import org.upnp.ResType;

import upnp.UPnPContainer;
import upnp.UPnPTypes;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

/**
 * SESimControlPoint
 * 
 * Class which represents the logic of the SESim control point
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class SESimControlPoint extends ControlPoint implements DeviceChangeListener {
	private Hashtable<String, Device> mediaServerList_ = null;
	
	private final static String CONTENT_DIRECTORY = "urn:schemas-upnp-org:service:ContentDirectory:1";
	// Max number of returned items for browse requests
	private final static int MAX_REQUEST_COUNT = 250;
	
	/**
	 * Constructor
	 */
	public SESimControlPoint() {
		mediaServerList_ = new Hashtable<String, Device>();
		addDeviceChangeListener(this);

		this.setNMPRMode(true);
		UPnP.setEnable(UPnP.USE_ONLY_IPV4_ADDR);
	}
	
	/**
	 * Method for printing a list with all known devices
	 */
	public void printDeviceList() {
		Enumeration<String> keys = mediaServerList_.keys();
		while (keys.hasMoreElements()) {
			Device dev = mediaServerList_.get(keys.nextElement());
			String devName = dev.getFriendlyName();
			String location = dev.getLocation();
			System.out.println("Device = " + devName + "; Location: " + location);
			
			System.out.println("Printing Service List:");
			ServiceList sl = dev.getServiceList();
			for (int i = 0; i < sl.size(); i++) {
				Service svc = (Service)sl.get(i);
				System.out.println("[" + i + "] = " + svc.getServiceType());
			}
		}
	}
	
	/**
	 * Method for retrieving the service type
	 * @return String with the service type
	 */
	public String getServiceType() {
		return CONTENT_DIRECTORY;
	}
	
	public boolean start() {
		return super.start();
	}
	
	public boolean stop() {
		return super.stop();
	}

	public void deviceAdded(Device dev) {
		if((dev.getDeviceType().indexOf("device:MediaServer") >= 0) ||
			(dev.getDeviceType().indexOf("device:ContentDirectory") >= 0)) {
			mediaServerList_.put(dev.getUDN(), dev);
			//System.out.println("UPnP: Device added!");
		}
		
		//printDeviceList();
	}

	public void deviceRemoved(Device dev) {
		if (mediaServerList_.containsKey(dev.getUDN())) {
			mediaServerList_.remove(dev.getUDN());
			//System.out.println("UPnP: Device removed!");
		}
		
		//printDeviceList();
	}
	
	/**
	 * Method for retrieving the number of registered devices
	 * @return int with the number of registered devices
	 */
	public int getNumberOfRegisteredDevices() {
		return mediaServerList_.size();
	}
	
	/**
	 * Method for retrieving a hashtable with the server list
	 * @return Hashtable<String, Device> with the UDN and the device
	 */
	public Hashtable<String, Device> getServerList() {
		return mediaServerList_;
	}
	
	/**
	 * Method for retrieving a vector with the server list
	 * @return Vector<Device> with the registered devices
	 */
	public Vector<Device> getServerListAsVector() {
		Vector<Device> retVal = new Vector<Device>();
		Enumeration<String> keys = mediaServerList_.keys();
		while (keys.hasMoreElements()) {
			retVal.add(mediaServerList_.get(keys.nextElement()));			
		}
	
		return retVal;
	}
	
	/**
	 * Method for retrieving the content of a server
	 * @param dev Device from which to retrieve
	 * @param objectID in with the ID of the object to retrieve
	 * @param topContainer UPnPContainer which act as parent container
	 * @return UPnPContainer with the content
	 */
	public UPnPContainer getServerContentList(Device dev, int objectID, UPnPContainer topContainer) {
		if (dev == null) {
			System.out.println("A device has to be given!");
			return null;
		}
		
		// retrieve all elements from the server (in current opened view)
		Action action = dev.getAction("Browse");
		if (action == null) {
			System.out.println("unable to browse device!");
			return null;
		}
		
		int requestedCount = MAX_REQUEST_COUNT;

		int totalMatches = 0;
		int totalReturnedMatches = 0;

		UPnPContainer retVal = null;
		do {
			action.setArgumentValue("ObjectID", objectID );
			action.setArgumentValue("BrowseFlag", "BrowseDirectChildren" );
			/*action.setArgumentValue("Filter", "" );
		    action.setArgumentValue("StartingIndex",
		                              Integer.toString(totalReturnedMatches) );
		    action.setArgumentValue("RequestedCount",
		                              Integer.toString(requestedCount) );
		    action.setArgumentValue("SortCriteria", "" );*/

		    boolean status = action.postControlAction();
		    if(status == false ) {
		    	System.out.println("postControlAction failed!");
		        return null;
		    }
		      
	        // Returned args are:  Result, NumberReturned, TotalMatches, UpdateId
		    ArgumentList outArgList = action.getOutputArgumentList();
		    Argument resultArg = outArgList.getArgument("Result");
		    Argument numberReturnedArg = outArgList.getArgument("NumberReturned");
		    Argument totalMatchesArg = outArgList.getArgument("TotalMatches");

		    if (resultArg == null || numberReturnedArg == null ||
		        totalMatchesArg == null) {
		        System.out.println("Error reading browse arguments" );
		        return null;
		    }
		        
		   	totalMatches = totalMatchesArg.getIntegerValue();
		    totalReturnedMatches += numberReturnedArg.getIntegerValue();
		    if (totalMatches == 0) {
		       System.out.println("browse - no objects found!!!");
		       return null;
		    }

		    requestedCount = totalMatches - totalReturnedMatches;
		    if (requestedCount > MAX_REQUEST_COUNT)
		       requestedCount = MAX_REQUEST_COUNT;

		    String result = resultArg.getValue();
		    retVal = extractContent(dev, result, topContainer);		    
		 } while(totalReturnedMatches < totalMatches);
		
		//System.out.println(retVal);
		
		return retVal;
	}
	
	/**
	 * Method for parsing a DIDL-Lite description
	 * @param didl String with the DIDL-Lite
	 * @return DIDLLite containing the parsed DIDL-Lite
	 * @throws JAXBException if a parse error occurred
	 */
	private DIDLLite parseDIDLLiteDescription(String didl) throws JAXBException {
		JAXBContext jaxbContext;
		try {
			jaxbContext = JAXBContext.newInstance(DIDLLite.class);
			Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
			JAXBElement<DIDLLite> elements = unmarshaller.unmarshal(new StreamSource(new StringReader(didl)), DIDLLite.class);
			return elements.getValue();
		} catch (JAXBException e) {
			throw new JAXBException("Parsing error: " + e.getMessage());
		} catch (Exception e) {
			throw new JAXBException("Unexpected error: " + e.getMessage());
		}
	}
	
	/**
	 * Method for extracting content of a DIDL-Lite
	 * @param dev Device from which to extract
	 * @param didl String with the DIDL-Lite
	 * @param topContainer UPnPContainer acting as parent container
	 * @return UPnPContainer with the extracted content
	 */
	private UPnPContainer extractContent(Device dev, String didl, UPnPContainer topContainer) {
		try {
			if (topContainer == null)
				topContainer = new UPnPContainer(UPnPTypes.UPNP_TOP, 0, dev.getFriendlyName());
			
			Vector<UPnPContainer> parentContainers = new Vector<UPnPContainer>();
			Vector<UPnPContainer> childContainers = new Vector<UPnPContainer>();
			
			//System.out.println("DEBUG: " + didl);
			DIDLLite lite = parseDIDLLiteDescription(didl);
			List<Object> itemscontdesc = lite.getItemsAndContainersAndDescs();
			for (int i = 0; i < itemscontdesc.size(); i++) {
				Object item = itemscontdesc.get(i);
				
				UPnPContainer container = null;				
				List<JAXBElement<?>> list = new ArrayList<JAXBElement<?>>();
				int type = 0;
				
				if (item instanceof ItemType) {
					// e.g., elements like video, image etc.
					ItemType it = (ItemType)item;
					
					type = UPnPTypes.UPNP_ITEM;
					container = new UPnPContainer(UPnPTypes.UPNP_ITEM, Integer.parseInt(it.getId()), it.getTitle().getValue());
					container.setRefid(it.getRefID() != null ? Integer.parseInt(it.getRefID()) : 0);
					container.setParentID(it.getParentID() != null ? Integer.parseInt(it.getParentID()) : 0);
					
					//System.out.println("id: " + it.getId() + "; parentID: " + it.getParentID());
					//System.out.println("refid: " + it.getRefID() + "; title: " + it.getTitle().getValue());
					
					list = (List<JAXBElement<?>>)it.getSearchClassesAndCreateClassesAndWriteStatuses();
					
				}
				else if (item instanceof ContainerType) {
					// e.g., elements like Decade, Video, Image. NOTE: this can include ItemTypes!
					ContainerType ct = (ContainerType)item;
					
					type = UPnPTypes.UPNP_CONTAINER;
					container = new UPnPContainer(UPnPTypes.UPNP_CONTAINER, Integer.parseInt(ct.getId()), ct.getTitle().getValue());
					container.setChildCount(ct.getChildCount());
					container.setParentID(ct.getParentID() != null ? Integer.parseInt(ct.getParentID()) : 0);
					
					//System.out.println("id: " + ct.getId() + "; parentID: " + ct.getParentID());
					//System.out.println("childCount: " + ct.getChildCount() + "; title: " + ct.getTitle().getValue());
					
					list = (List<JAXBElement<?>>)ct.getSearchClassesAndCreateClassesAndWriteStatuses();
				}
				// currently ignored!
				else if (item instanceof DescType)
					;
				else {
					System.out.println("Unsupported type!");
					return null;
				}
							
				for (int j = 0; j < list.size(); j++) {
					Object elem = list.get(j).getValue();
					if (elem instanceof ResType) {
						ResType rt = (ResType)elem;
						container.addResource(rt);
						//System.out.println("Resource: " + rt.getValue());
					}
					//  currently not needed
					else if (elem instanceof PersonWithRoleType) {
						;
					}
					else if (elem instanceof GenreType) {
						;
					}
					else if (elem instanceof ClassType) {
						;
					}
					else if (elem instanceof DescType) {
						;
					}
					else
						;//System.out.println(elem);
				}
				
				if (type == UPnPTypes.UPNP_ITEM)
					childContainers.add(container);
				else if (type == UPnPTypes.UPNP_CONTAINER)
					parentContainers.add(container);
			}
			
			topContainer.setSubElements(parentContainers);
			topContainer.addSubElements(childContainers);
			return topContainer;
		} catch (JAXBException e) {
			e.printStackTrace();
			return null;
		}
	}
}
