/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package timer;

import interfaces.DeviceDefinition;

import java.awt.Color;
import java.text.DecimalFormat;

import javax.swing.border.EtchedBorder;

import device.AirConditionerDevice;

import app.SESimGUI;

/**
 * Timer task for AirConditioner device
 * 
 * manages effects of the air-conditioner device
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class AirConditionerTask extends java.util.TimerTask {
	private double timeCounter = -1;
	private double timeCounterStop = -1;
	private SESimGUI gui_ = null;
	private AirConditionerDevice device_ = null;
	private float intensity_ = 0.0f;
	private float finalIntensity_ = 0.0f;
	private long fade_ = 0;
	private float duration_ = -1.0f;
	private float decSeconds = DeviceDefinition.DECREASETIME;
	private float stepping_ = 0.0f;
	
	/**
	 * Constructor
	 * 
	 * @param intensity initial intensity
	 * @param duration time for the effect to run
	 * @param cycles cycles needed for fade
	 * @param oldIntensityValue intensity of the last run if not deactivated in between
	 * @param gui SESimGUI for output
	 * @param device AirConditionerDevice for feedback
	 */
	public AirConditionerTask(float intensity, float duration, long cycles, float oldIntensityValue, SESimGUI gui, AirConditionerDevice device) {
		finalIntensity_ = intensity;
		duration_ = duration;
		fade_ = cycles;
		gui_ = gui;
		device_ = device;
		
		if (oldIntensityValue > 0)
			intensity_ = oldIntensityValue;
		else
			intensity_ = intensity;
		
		if (fade_ > 0) {
			if (oldIntensityValue > 0)
				stepping_ = (finalIntensity_-oldIntensityValue)/fade_;
			else
				stepping_ = finalIntensity_/fade_;
		}
	}

	/**
	 * Thread running method
	 */
	public void run() {
		if (timeCounterStop == -1) {
			timeCounter = gui_.panVideo.getCurrentVideoTime();
			if (fade_ <= 0) {
				gui_.panAirConditioner.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui_.panAirConditioner.drawText(String.valueOf(new DecimalFormat("#.##").format(intensity_)));
				
				if (duration_ > 0) {
					duration_ = duration_ - decSeconds;
					if (duration_ <= 0) {
						gui_.panAirConditioner.setBorder(new EtchedBorder());
						gui_.panAirConditioner.drawText("");
						device_.setActivate(false);
						this.cancel();
					}
				}
			}
			else if (fade_ > 0) {
				gui_.panAirConditioner.setBorder(new EtchedBorder(Color.RED, Color.RED));
				gui_.panAirConditioner.drawText(String.valueOf(new DecimalFormat("#.##").format(intensity_)));
				
				fade_--;
				intensity_ = intensity_ + stepping_;
				
				if (duration_ > 0)
					duration_ = duration_ - decSeconds;
			}
		}
	}
	
	/**
	 * Method for pausing the timer
	 */
	public void pause() {
		timeCounterStop = timeCounter;
	}
	
	/**
	 * Method for resuming the timer
	 */
	public void resume() {
		timeCounter = timeCounterStop;
		timeCounterStop = -1;
	}
}
