/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package timer;

import interfaces.DeviceDefinition;

import java.awt.Color;
import javax.swing.border.EtchedBorder;
import device.LightDevice;
import lib.ClassificationScheme;
import lib.HelperFunctions;
import lib.Histogram;
import app.DeviceInfoWindow;
import app.SESimGUI;

/**
 * Timer task for Light device
 * 
 * manages effects of the light device
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */

// NOTE: light can differ to real implementations!!!
// TODO: split autocolor coloring into three parts as in the Sensory Effect Media Player (SEMP)
public class LightTask extends java.util.TimerTask {
	private double timeCounter = -1.0;
	private double timeCounterStop = -1.0;
	private SESimGUI gui_ = null;
	private LightDevice device_ = null;
	private DeviceInfoWindow diw_ = null;
	private float intensity_ = -1.0f;
	private float finalIntensity_ = -1.0f;
	private long fade_ = 0;
	private float duration_ = -1.0f;
	private float decSeconds = DeviceDefinition.DECREASETIME;
	private float stepping_ = 0.0f;	
	private Color color_ = Color.WHITE;
	private String location_ = null;
	private boolean autoColor_ = false;
	
	/**
	 * Constructor
	 * 
	 * @param diw DeviceInfoWindows instance for updating info
	 * @param color Color with the color to display
	 * @param autoColor true if automatic color extraction should be used else false
	 * @param intensity initial intensity
	 * @param duration time for the effect to run
	 * @param cycles cycles needed for fade
	 * @param range upper and lower bound of intensity range
	 * @param location location of the device
	 * @param oldIntensityValue intensity of the last run if not deactivated in between
	 * @param gui SESimGUI for output
	 * @param device LightDevice for feedback
	 */
	public LightTask(DeviceInfoWindow diw, Color color, boolean autoColor, float intensity, float duration, long cycles, float[] range, String location, float oldIntensityValue, SESimGUI gui, LightDevice device) {
		color_ = color;
		// update start color to already match intensity
		// if no intensity is given some parameters are useless
		if (intensity > -1.0f) {
			float[] hsl = HelperFunctions.fromRGB(color_);
			color_ = HelperFunctions.adjustLuminance(hsl, intensity_);
			fade_ = cycles;	
			finalIntensity_ = (100.0f/range[1]*intensity); // 100% / upper bound * desired intensity = percent of luminance
			
			if (oldIntensityValue > 0)
				intensity_ = oldIntensityValue;
			else
				intensity_ = intensity;
			
			if (fade_ > 0) {
				if (oldIntensityValue > 0)
					stepping_ = (finalIntensity_-oldIntensityValue)/fade_;
				else
					stepping_ = finalIntensity_/fade_;
			}
		}
		
		autoColor_ = autoColor;		
		duration_ = duration;
		location_ = location;
		gui_ = gui;
		device_ = device;
		diw_ = diw;		
	}
	
	/**
	 * Thread running method
	 */
	public void run() {
		if (timeCounterStop == -1) {
			timeCounter = gui_.panVideo.getCurrentVideoTime();
			
			// if no autocolor is used fade, duration etc. will be ignored! That means
			// if autocolor is used the whole movie will use autocolor (for simplicity)
			// Note: other software products can handle this in another way!!!
			if (!autoColor_) {
				if (fade_ <= 0) {				
					if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
						gui_.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightLeft.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
					   	gui_.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackLeft.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
					   	gui_.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackMiddle.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);					
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
					   	gui_.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackRight.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					   	gui_.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightRight.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
						gui_.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	
						gui_.panLightLeft.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
						gui_.panLightBackLeft.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
						gui_.panLightBackMiddle.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
						gui_.panLightBackRight.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					   	gui_.panLightRight.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					
							
					if (duration_ > 0) {
						duration_ = duration_ - decSeconds;
						if (duration_ <= 0) {
							if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
								gui_.panLightLeft.setBorder(new EtchedBorder());
								gui_.panLightLeft.drawText("", "", "");
								diw_.changeValue("LightDevice Left", "");
							}
							else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
							   	gui_.panLightBackLeft.setBorder(new EtchedBorder());
								gui_.panLightBackLeft.drawText("");
							   	diw_.changeValue("LightDevice LeftBack", "");
							}
							else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
							   	gui_.panLightBackMiddle.setBorder(new EtchedBorder());
								gui_.panLightBackMiddle.drawText("");
							   	diw_.changeValue("LightDevice MiddleBack", "");			
							}
							else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
							   	gui_.panLightBackRight.setBorder(new EtchedBorder());
								gui_.panLightBackRight.drawText("");
								diw_.changeValue("LightDevice RightBack", "");
							}
							else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
							   	gui_.panLightRight.setBorder(new EtchedBorder());
							   	gui_.panLightRight.drawText("", "", "");					
								diw_.changeValue("LightDevice Right", "");
							}
							else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
								gui_.panLightLeft.setBorder(new EtchedBorder());
							   	gui_.panLightBackLeft.setBorder(new EtchedBorder());
							   	gui_.panLightBackMiddle.setBorder(new EtchedBorder());
							   	gui_.panLightBackRight.setBorder(new EtchedBorder());
							   	gui_.panLightRight.setBorder(new EtchedBorder());
							   	
							   	gui_.panLightLeft.drawText("", "", "");
								gui_.panLightBackLeft.drawText("");
								gui_.panLightBackMiddle.drawText("");
								gui_.panLightBackRight.drawText("");
							   	gui_.panLightRight.drawText("", "", "");
							   	
							   	diw_.changeValue("LightDevice Left", "");
							   	diw_.changeValue("LightDevice LeftBack", "");
							   	diw_.changeValue("LightDevice MiddleBack", "");
								diw_.changeValue("LightDevice RightBack", "");					
								diw_.changeValue("LightDevice Right", "");
							}
							
							device_.setActivate(false);
							this.cancel();
						}
					}
				}
				else if (fade_ > 0) {
					if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_LEFT)) {
						gui_.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightLeft.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERLEFT)) {
					   	gui_.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackLeft.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTER)) {
					   	gui_.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackMiddle.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);					
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_CENTERRIGHT)) {
					   	gui_.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
						gui_.panLightBackRight.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_X_RIGHT)) {
					   	gui_.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightRight.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					else if (location_.startsWith(ClassificationScheme.LOCATIONURIBASE + ClassificationScheme.LOC_EVERYWHERE)) {
						gui_.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	gui_.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
					   	
						gui_.panLightLeft.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
						gui_.panLightBackLeft.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
						gui_.panLightBackMiddle.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
						gui_.panLightBackRight.drawText("R" + color_.getRed() + "G" + color_.getGreen() + "B" + color_.getBlue(), 14);
					   	gui_.panLightRight.drawText("R" + color_.getRed(), "G" + color_.getGreen(), "B" + color_.getBlue());
					}
					
					fade_--;
					intensity_ = intensity_ + stepping_;
					
					if (duration_ > 0)
						duration_ = duration_ - decSeconds;
					
					float[] hsl = HelperFunctions.fromRGB(color_);
					color_ = HelperFunctions.adjustLuminance(hsl, intensity_);
				}				
			}
			else {
				gui_.panLightLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
			   	gui_.panLightBackLeft.setBorder(new EtchedBorder(Color.RED, Color.RED));
			   	gui_.panLightBackMiddle.setBorder(new EtchedBorder(Color.RED, Color.RED));
			   	gui_.panLightBackRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
			   	gui_.panLightRight.setBorder(new EtchedBorder(Color.RED, Color.RED));
			   	
				autoColorCalculation();
			}
		}
	}
	
	/**
	 * Method for pausing the timer
	 */
	public void pause() {
		timeCounterStop = timeCounter;
	}
	
	/**
	 * Method for resuming the timer
	 */
	public void resume() {
		timeCounter = timeCounterStop;
		timeCounterStop = -1.0;
	}
	
	/**
	 * Method for calculating the auto color
	 */
	private void autoColorCalculation() {
		// TODO: color should be for every light different, needs to be implemented sometimes
		java.awt.image.BufferedImage img = gui_.panVideo.getFrame(); 
		if (img != null) {
			
			Histogram histo = new Histogram();
			histo.extract(img);
		    int[] avgColor = histo.getAverageColor();
		    
		    Color color = null;
		    if (intensity_ > -1.0f) {
		    	float[] hsl = HelperFunctions.fromRGB(new Color(avgColor[0], avgColor[1], avgColor[2]));
		    	color = HelperFunctions.adjustLuminance(hsl, intensity_);
		    }
		    else {
		    	color = new Color(avgColor[0], avgColor[1], avgColor[2]);
		    }
		    
			gui_.panLightLeft.drawText("R" + color.getRed() , "G" + color.getGreen(), "B" + color.getBlue());
			gui_.panLightBackLeft.drawText("R" + color.getRed() + "G" + color.getGreen() + "B" + color.getBlue(), 14);
			gui_.panLightBackMiddle.drawText("R" + color.getRed() + "G" + color.getGreen() + "B" + color.getBlue(), 14);
			gui_.panLightBackRight.drawText("R" + color.getRed() + "G" + color.getGreen() + "B" + color.getBlue(), 14);
		   	gui_.panLightRight.drawText("R" + color.getRed(), "G" + color.getGreen(), "B" + color.getBlue());
		}
	}
}
