/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package upnp;
import java.util.Vector;

import org.upnp.ResType;

/**
 * UPnPContainer
 * 
 * Class which represents a container for a UPnP entry (e.g., Container, Item)
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class UPnPContainer {
	private int type_ = UPnPTypes.UPNP_CONTAINER;
	private int id_ = 0;
	private int refid_ = 0;
	private long childCount_ = 0;
	private int parentID_ = 0;
	private String title_ = "";
	private Vector<ResType> resources_ = null;
	private Vector<UPnPContainer> subElements_ = null;
	
	/**
	 * Constructor
	 * 
	 * @param type UPnP type
	 * @param id ID of the container
	 * @param title Title of the entry
	 */
	public UPnPContainer(int type, int id, String title) {
		type_ = type;
		id_ = id;
		title_ = title;
	}
	
	/**
	 * Method for retrieving the type
	 * @return int with type
	 */
	public int getType() {
		return type_;
	}
	
	/**
	 * Method for retrieving the ID
	 * @return int with id
	 */
	public int getId() {
		return id_;
	}
	
	/**
	 * Method for retrieving the title
	 * @return String with title
	 */
	public String getTitle() {
		return title_;
	}
	
	/**
	 * Method for retrieving the children of the container
	 * @return Vector<UPnPContainer> with the children
	 */
	public Vector<UPnPContainer> getSubElements() {
		return subElements_;
	}
	
	/**
	 * Method for setting the children of the container
	 * @param subElements Vector<UPnPContainer> with the children
	 */
	public void setSubElements(Vector<UPnPContainer> subElements) {
		this.subElements_ = subElements;
	}
	
	/**
	 * Method for adding a set of children to the container
	 * @param subElements Vector<UPnPContainer> with the children to add
	 */
	public void addSubElements(Vector<UPnPContainer> subElements) {
		subElements_.addAll(subElements);
	}
	
	/**
	 * Method for retrieving information about sub elements
	 * @return true if there are children else false
	 */
	public boolean hasSubElements() {
		return subElements_ == null ? false : true;
	}
	
	/**
	 * Method for adding a single child to the container
	 * @param subElement UPnPContainer with the child
	 */
	public void addSubElement(UPnPContainer subElement) {
		if (subElements_ == null)
			subElements_ = new Vector<UPnPContainer>();
		
		if (!subElements_.contains(subElement))
			subElements_.add(subElement);
	}

	/**
	 * Method for retrieving the reference ID
	 * @return int with reference id
	 */
	public int getRefid() {
		return refid_;
	}

	/**
	 * Method for setting the reference ID
	 * @param refid int with the id
	 */
	public void setRefid(int refid) {
		this.refid_ = refid;
	}

	/**
	 * Method for retrieving the child count
	 * @return long with child count
	 */
	public long getChildCount() {
		return childCount_;
	}

	/**
	 * Method for setting the child count
	 * @param childCount long with the child count
	 */
	public void setChildCount(long childCount) {
		this.childCount_ = childCount;
	}

	/**
	 * Method for retrieving the parent ID
	 * @return int with the parent ID
	 */
	public int getParentID() {
		return parentID_;
	}

	/**
	 * Method for setting the parent ID
	 * @param parentID int with the parent ID
	 */
	public void setParentID(int parentID) {
		this.parentID_ = parentID;
	}

	/**
	 * Method for retrieving associated resources
	 * @return Vector<ResType> with resources
	 */
	public Vector<ResType> getResources() {
		return resources_;
	}
	
	/**
	 * Method for adding a resource to the container
	 * @param resource ResType with the resource
	 */
	public void addResource(ResType resource) {
		if (resources_ == null)
			resources_ = new Vector<ResType>();
		
		resources_.add(resource);
	}
	
	public String toString() {
		String retVal = "UPnPContainer\n==========\n";
		
		if (type_ == UPnPTypes.UPNP_TOP)
			retVal += "Type: Root\n";
		else if (type_ == UPnPTypes.UPNP_CONTAINER)
			retVal += "Type: Container\n";
		else
			retVal += "Type: Item\n";		
		
		retVal += "Title: " + title_ + "\n";
		retVal += "ID: " + id_ + "\n";
		retVal += "RefID: " + refid_ + "\n";
		retVal += "ChildCount: " + childCount_ + "\n";
		retVal += "ParentID: " + parentID_ + "\n";
		
		retVal += "Resources:\n------\n";
		for (int i = 0; resources_ != null && i < resources_.size(); i++)
			retVal += "Mimetype: " + resources_.elementAt(i).getProtocolInfo() + "; Path: " + 
			resources_.elementAt(i).getValue() + "\n";
		
		retVal += "SubElements:\n--------\n";
		for (int i = 0; subElements_ != null && i < subElements_.size(); i++)
			retVal += subElements_.elementAt(i).toString();
		
		retVal += "END of UPnPContainer\n\n";
		
		return retVal;
	}
}
