/*
 * This file is part of SESim.
 *
 * SESim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SESim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SESim.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package xml;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.stream.StreamSource;

import lib.HelperFunctions;
import lib.SEMDefinitions;

import org.iso.mpeg.mpegv.sedl.EffectBaseType;
import org.iso.mpeg.mpegv.sev.FogType;
import org.iso.mpeg.mpegv.sedl.GroupOfEffects;
import org.iso.mpeg.mpegv.sev.LightType;
import org.iso.mpeg.mpegv.sedl.SEM;
import org.iso.mpeg.mpegv.sedl.SEMBaseType;
import org.iso.mpeg.mpegv.sev.ScentType;
import org.iso.mpeg.mpegv.sev.TemperatureType;
import org.iso.mpeg.mpegv.sev.VibrationType;
import org.iso.mpeg.mpegv.sev.SprayingType;
import org.iso.mpeg.mpegv.sev.WindType;
import interfaces.DeviceDefinition;

/**
 * SEMParser
 * 
 * Class for handling the SEM content in the XML file
 * (partly FDIS conform (not all parts implemented!))
 * 
 * Note: the parser changes the input XML to an internal
 * string based structure! (will maybe be reworked ;-))
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class SEMParser {
	private static Vector<String> buffer_ = null;
	private double curTime = -1;
	private long timeScale = 1;
	
	/**
	 * Constructor
	 */
	public SEMParser() {
		buffer_ = new Vector<String>();
	}
	
	/**
	 * Parses a SEM description
	 * @param fileToParse String representing the path to the SEM.
	 * @return Vector<String> with parsed information.
	 * @throws JAXBException if a parse error occured.
	 * @throws IOException if an I/O error occured.
	 */
	public Vector<String> parse(String fileToParse) throws JAXBException, IOException {
		JAXBContext jaxbContext;
		try {
			File file = new File(fileToParse);
			if (!file.exists() || !file.canRead())
				throw new IOException("IO error: file does not exist or cannot be read!");
			
			jaxbContext = JAXBContext.newInstance(SEM.class);
			Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
			JAXBElement<SEM> jSem = unmarshaller.unmarshal(new StreamSource(file), SEM.class);
			SEM sem =  jSem.getValue();
			
			Map<QName, String> semAttributes = sem.getOtherAttributes();
			if (semAttributes != null) {
				for (QName key : semAttributes.keySet()) {
					if (key.getLocalPart().equals(SEMDefinitions.TIMESCALE)) {
						timeScale = Long.valueOf(semAttributes.get(key));
					}
				}
			}
			
			List<SEMBaseType> semContent = sem.getDeclarationsAndGroupOfEffectsAndEffects();
			if (semContent != null) {
				for (int i = 0; i < semContent.size(); i++) {
					SEMBaseType sbt = semContent.get(i);
					if (sbt instanceof EffectBaseType) {
						handleEffect((EffectBaseType)sbt);
						
						curTime = -1;
					}
					else if (sbt instanceof GroupOfEffects) {
						GroupOfEffects goe = (GroupOfEffects)sbt;
										
						//NOTE: attributes from groupofeffects are currently ignored! (e.g., alt, duration, ...)
						Map<QName, String> goeAttributes = goe.getOtherAttributes();
						if (goeAttributes != null) {
							for (QName key : goeAttributes.keySet()) {
								if (key.getLocalPart().equals(SEMDefinitions.PTS)) {
									double lVal = Long.valueOf(goeAttributes.get(key));
									curTime = lVal/timeScale;
								}
								else if (key.getLocalPart().equals(SEMDefinitions.ANCHORELEMENT)) {
									// currently not needed
								}
							}
						}
								
						List<SEMBaseType> groupEffects = goe.getEffectsAndReferenceEffects();
						if (groupEffects != null) {
							for (SEMBaseType goeSBT : groupEffects) {
								if (goeSBT instanceof EffectBaseType) {
									handleEffect((EffectBaseType)goeSBT);
								}
								// reference effect not supported (for now!)
							}
						}
						
						curTime = -1;
					}
				}
			}			
		} catch (JAXBException e) {
			throw new JAXBException(e.getMessage());
		} catch (Exception e) {
			throw new JAXBException(e.getMessage());
		}
		
		return getSEMContent();
	}
	
	/**
	 * Method for handling effects, e.g., extracting effect attributes
	 * @param ebt EffectBaseType with the effect to be handled
	 */
	private void handleEffect(EffectBaseType ebt) {
		String type = "";
		String attributes = "";
		String time = "";
		
		if (ebt instanceof WindType)
			type = DeviceDefinition.FAN;
		else if (ebt instanceof VibrationType)
			type = DeviceDefinition.VIBRATOR;
		else if (ebt instanceof LightType)
			type = DeviceDefinition.LIGHT;
		else if (ebt instanceof TemperatureType)
			type = DeviceDefinition.AIRCONDITIONER;
		else if (ebt instanceof SprayingType)
			type = DeviceDefinition.WATERSPRAYER;
		else if (ebt instanceof ScentType)
			type = DeviceDefinition.PERFUMER;
		else if (ebt instanceof FogType)
			type = DeviceDefinition.FOG;
		
		if (ebt.isActivate() != null)
			attributes += SEMDefinitions.ACTIVATE + "=" + ebt.isActivate() + ", ";
		if (ebt.getAlt() != null)
			attributes += SEMDefinitions.ALT + "=" + ebt.getAlt() + ", ";
		if (ebt.getAutoExtraction() != null)
			attributes += SEMDefinitions.AUTOEXTRACTION + "=" + ebt.getAutoExtraction().value() + ", ";
		if (ebt.getDuration() != null)
			attributes += SEMDefinitions.DURATION + "=" + ebt.getDuration().longValue()/timeScale + ", ";
		if (ebt.getFade() != null)
			attributes += SEMDefinitions.FADE + "=" + ebt.getFade().longValue()/timeScale + ", ";
		if (ebt.getLocation() != null)
			attributes += SEMDefinitions.LOCATION + "=" + ebt.getLocation() + ", ";
		if (ebt.getPriority() != null)
			attributes += SEMDefinitions.PRIORITY + "=" + ebt.getPriority() + ", ";
		
		Map<QName, String> effectAttributes = ebt.getOtherAttributes();
		if (effectAttributes != null) {
			for (QName key : effectAttributes.keySet()) {
				if (key.getLocalPart().equals(SEMDefinitions.PTS)) {
					double lVal = Long.valueOf(effectAttributes.get(key));
					lVal = lVal/timeScale;
					
					double seconds = (lVal%60);
					lVal = lVal/60;
					int minutes = (int)(lVal%60);
					lVal = lVal/60;
					int hours = (int)lVal;
					
					if (hours < 10)
						time = "0" + hours + ":";
					else
						time = "" + hours + ":";
					
					if (minutes < 10)
						time += "0" + minutes + ":";
					else
						time += "" + minutes + ":";
					
					if (seconds < 10)
						time += "0" + HelperFunctions.round(seconds, 1);
					else
						time += "" + HelperFunctions.round(seconds, 1);
				}
				else if (key.getLocalPart().equals(SEMDefinitions.ANCHORELEMENT)) {
					// not interesting for now within effects
				}
				else {
					if (!attributes.equals("") && !attributes.endsWith(", "))
						attributes += ", ";
						
					attributes += key.getLocalPart() + "=" + effectAttributes.get(key);
				}
			}
		}
		
		// we need to remove the last ","
		if (!attributes.equals("") && attributes.endsWith(", "))
			attributes = attributes.substring(0, attributes.lastIndexOf(","));
		
		if (curTime != -1) {
			double cTime = curTime;
			double seconds = cTime%60;
			cTime = cTime/60;
			int minutes = (int)(cTime%60);
			cTime = cTime/60;
			int hours = (int)cTime;
			
			if (hours < 10)
				time = "0" + hours + ":";
			else
				time = "" + hours + ":";
			
			if (minutes < 10)
				time += "0" + minutes + ":";
			else
				time += "" + minutes + ":";
			
			if (seconds < 10)
				time += "0" + HelperFunctions.round(seconds, 1);
			else
				time += "" + HelperFunctions.round(seconds, 1);
		}
		
		buffer_.add(time + "=" + type + "(" + attributes + ")");
	}
		
	/**
	 * Method for retrieving the SEM content
	 * @return Vector<String> with the content
	 */
	public Vector<String> getSEMContent() {
		if (buffer_.isEmpty())
			return null;
		
		return buffer_;
	}
}